<?php
$ems_cfg = array(
    'UserID'        =>  '',     // The creditors's user id from the registration process at the scheme operator
    'PIN'           =>  '',     // PIN
    'Id'            =>  '',     // Creditor ID
    'Cdtr'          =>          // Information about the creditor
    array(
        'Nm'        =>  '',     // Name
        'Ctry'      =>  '',     // ISO 3166 country code
        'AdrLine'   =>  array ( // Max. 2 lines!
            '',                 // 70 characters per line (Street)
            ''                  // optional second line, 70 chars max. (City)
        )
    ),

    'System'        => 'test',   // Set to 'test' for test environment, empty for production
    'Logfile'       => 'log.txt',// Writes transactions to this logfile, set empty for no log

    // Defaults for EMS_Initiate, for details, see EMS_Initiate description
    'ReturnUrl'     => 'https://'.$_SERVER['HTTP_HOST'].dirname($_SERVER['REQUEST_URI']).'/rsp.php',
    'Lang'          => 'DE',
    'ExpirationTime'=> 60,
    'LclInstrm'     => 'CORE',
    'Ocrncs'        => 'RCUR'
);

?>
