<?php
/* 
 * Sample response check for EMS 
 */
include_once('ems_functions.inc.php');

session_start();
$ret = EMS_Status($_SESSION['emsReq']);
if ($ret['ErrorMsg']) die($ret['ErrorMsg']);
echo "EMS Status of transaction is: ".$ret['Status'];
?>
